"""
w2lapp.cnf.py: read configuration data

web2ldap - a web-based LDAP Client,
see http://www.web2ldap.de for details

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: cnf.py,v 1.35 2005/02/23 17:26:12 michael Exp $
"""

import signal,w2lapp.schema,ldap,ldap.schema,ldapsession
from ldap.cidict import cidict

import web2ldapcnf,web2ldapcnf.hosts,web2ldapcnf.misc,web2ldapcnf.standalone,web2ldapcnf.fastcgi
from web2ldapcnf import misc,hosts,standalone,fastcgi

from ldapurl import LDAPUrl

class Web2LDAPConfig(cidict):

  def _normalizeKey(self,key):
    if isinstance(key,ldapsession.LDAPSession):
      if key.uri is None:
        result = '_'
      else:        
        result = '/'.join((str(key.uri),key.currentSearchRoot.encode(key.charset)))
    elif isinstance(key,LDAPUrl):
      key.scope = None
      result = str(key)
    else:
      key = key.strip()
      if key=='_':
        result = '_'
      else:
        try:
          lu = LDAPUrl(key)
        except ValueError:
          raise ValueError,"Configuration key MUST be valid LDAP URL or a single underscore '_'"
        else:
          lu.attrs = None
          lu.filterstr = None
          lu.scope = None
          lu.extensions = None
          result = str(lu)
    return result

  def __getitem__(self,key):
    return cidict.__getitem__(self,self._normalizeKey(key))

  def __delitem__(self,key,value):
    return cidict.__setitem__(self,self._normalizeKey(key))

  def __setitem__(self,key,value):
    return cidict.__setitem__(self,self._normalizeKey(key),value)

  def has_key(self,key):
    return cidict.has_key(self,self._normalizeKey(key))

  def GetParam(self,backend_key,param_key,default=''):
    lu_key = self._normalizeKey(backend_key or '_')
    try:
      return self[lu_key].__dict__[param_key]
    except KeyError:
      try:
        return self[str(LDAPUrl(dn=LDAPUrl(lu_key).dn))].__dict__[param_key]
      except KeyError:
        try:
          return self[LDAPUrl(lu_key).initializeUrl()].__dict__[param_key]
        except KeyError:
          try:
            return self['_'].__dict__[param_key]
          except KeyError:
            return default
    
ldap_def = Web2LDAPConfig(web2ldapcnf.hosts.ldap_def)
w2lapp.schema.parse_fake_schema(ldap_def)

def HUPSignalHandler(signum,frame):
  import w2lapp.session
  assert signum==signal.SIGHUP, \
    'Received wrong signal: Expected signal.SIGHUP, got %d' % (signum)
  reload(web2ldapcnf)
  reload(web2ldapcnf.hosts)
  reload(web2ldapcnf.misc)
  reload(web2ldapcnf.standalone)
  reload(web2ldapcnf.fastcgi)
  from web2ldapcnf import hosts,misc,standalone,fastcgi
  ldap_def = Web2LDAPConfig(web2ldapcnf.hosts.ldap_def)
  w2lapp.schema.parse_fake_schema(ldap_def)
  w2lapp.session.session.expireDeactivate = w2lapp.cnf.misc.session_remove
  w2lapp.session.session.expireRemove = w2lapp.cnf.misc.session_remove
  ldap._trace_level = web2ldapcnf.misc.ldap_trace_level

try:
  # Set signal handler for SIGHUP
  signal.signal(signal.SIGHUP,HUPSignalHandler)
except AttributeError:
  pass

def GetParam(ls,k,default=''):
  """
  Get a parameter determined by string-key k
  depending on current ls
  """
  return ldap_def.GetParam(ls,k,default)
